<?php

namespace Drupal\termcase\Commands;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drush\Commands\DrushCommands;
use Drush\Exceptions\UserAbortException;

/**
 * Class TermCaseCommands. The base class for Drush commands.
 */
class TermCaseCommands extends DrushCommands {

  /**
   * Returns the entity_type.manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Returns the config.factory service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Constructs a TermCaseCommands object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   Provides an interface for entity type managers.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   Defines the interface for a configuration object factory.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, ConfigFactoryInterface $config_factory) {
    parent::__construct();
    $this->entityTypeManager = $entity_type_manager;
    $this->configFactory = $config_factory;
  }

  /**
   * Update the case for taxonomy terms.
   *
   * @param string $vid
   *   Vocabulary machine name.
   *
   * @throws \Exception
   *
   * @usage drush term-case:update tags
   *   Update the case for taxonomy terms.
   *
   * @command term-case:update
   * @aliases tcu
   */
  public function updateTaxonomyCase($vid = '') {
    if (empty($vid)) {
      throw new \Exception('Vocabulary name is not specified.');
    }

    if (!$this->io()->confirm(dt('Are you sure you want to update all terms?'))) {
      throw new UserAbortException();
    }

    $term_storage = $this->entityTypeManager->getStorage('taxonomy_term');

    $query = $term_storage->getQuery();
    $query->condition('vid', explode(',', $vid), 'IN');
    $query->accessCheck(FALSE);
    $query->sort('tid');

    if ($tids = $query->execute()) {
      $batch = ['operations' => []];

      $config = $this->configFactory->get('termcase.settings');
      $vocabularies = $config->get('vocabularies', []);

      foreach ($term_storage->loadMultiple($tids) as $term) {
        $bundle = $term->bundle();
        $mode = !empty($vocabularies[$bundle]) ? $vocabularies[$bundle]['option'] : TERMCASE_NONE;

        $batch['operations'][] = ['\Drupal\termcase\TermCaseUpdateTerms::updateTerm', [$term, $mode]];
      }

      batch_set($batch);
      drush_backend_batch_process();

      $this->output()->writeln(dt('All selected taxonomy terms have been updated.'));
    }
    else {
      $this->output()->writeln(dt('No taxonomy terms found.'));
    }
  }

}
